/********************************************************
STM8S003K3 16MHz, IARSTM8_2.1+STM8S_StdPeriph_Lib_V2.1.0
*********************************************************/
#include "stm8s.h"
#include "iostm003.c"

#define A	  PD_ODR_ODR3
#define B	  PD_ODR_ODR2
#define C	  PD_ODR_ODR1
#define D	  PC_ODR_ODR7
#define E	  PC_ODR_ODR6
#define F  	PC_ODR_ODR5
#define G 	PE_ODR_ODR5
#define DP	PF_ODR_ODR4
#define N0	PA_ODR_ODR3
#define N1	PD_ODR_ODR7
#define N2	PD_ODR_ODR6
#define N3	PD_ODR_ODR5

#define key	PB_IDR_IDR7

#define pwm_on 	  do{pge=1;}while(0)
#define pwm_off 	do{pge=0;}while(0)
#define relay 		PB_ODR_ODR6

#define   	r_upr 20000 //Ohms
#define   	r_lwr 5100   //Ohms
#define   	u_max_adc ((500UL*(r_upr+r_lwr))/r_lwr) //=U_max*100, 24.6V*100
#define   	ust   1800 //output U val, 18V*100
const u16  	apv=(u16)((0x3ffUL*ust)/u_max_adc);
const u8   	arrv=0xff;

enum btn_states {prs, rls, hold} btn=rls;
enum dmodes {normal, started, undervoltage, overvoltage, lobat, sdn, sleep} dstate=started;

volatile u16 tim0,tim1,tim2,tim3;
u16 adc1val, adc0val;

volatile u8 ovl=0;
u8 pge=0;

u8 display_buffer[4]={0,0,0,1};
u8 dppos=0;

inline static void gpio_cfg()
{
  PA_DDR=PB_DDR=PC_DDR=PD_DDR=PE_DDR=PF_DDR=0xff; //conf as output
	PA_CR1=PB_CR1=PC_CR1=PD_CR1=PE_CR1=PF_CR1=0xff; //push-pull|pullup
  	//PA_CR2=PB_CR2=PC_CR2=PD_CR2=PE_CR2=PF_CR2=0;  //speed 2MHz|int disabled

	PB_DDR_DDR7=0; //btn
	PB_DDR_DDR3=0; //etr
	PB_DDR_DDR1=PB_DDR_DDR0=0; PB_CR1_C11=PB_CR1_C10 =0; //adc ch 0,1 pullup off
  PD_DDR_DDR0=0; //BKIN
}
inline static void clk_cfg()
{
  CLK_DeInit();
	CLK->CKDIVR = 0;     //16MHz
	CLK->PCKENR1 = 0x80; //only tim1&adc clk enabled
}

inline static void t1_cfg()
{
  TIM1->ARRH = 0;
  TIM1->ARRL = arrv;
  TIM1->CCR1L=TIM1->CCR2L=0;
  TIM1->CCR3L=TIM1->CCR4L=arrv;

	TIM1->CR1 = TIM1_COUNTERMODE_UP|TIM1_CR1_ARPE|TIM1_CR1_URS;
	TIM1->CR2 = TIM1_CR2_CCPC;
  TIM1->RCR = 0;

  TIM1->CCMR1=TIM1->CCMR2	= TIM1_OCMODE_PWM1|TIM1_CCMR_OCxPE|0x80;
	TIM1->CCMR3=TIM1->CCMR4	= TIM1_OCMODE_PWM2|TIM1_CCMR_OCxPE|0x80;
  TIM1->CCER1             = TIM1_CCER1_CC1E|TIM1_CCER1_CC2E;
	TIM1->CCER2             = TIM1_CCER2_CC3E|TIM1_CCER2_CC4E;
	TIM1->ETR	           	  = TIM1_ETR_ETP;   //active on falling edge
	TIM1->EGR               = TIM1_EGR_COMG;  //update CCER reg with COM event
	TIM1->IER               = TIM1_IER_UIE;   //update interrupt enable

	TIM1->BKR  = TIM1_BKR_MOE|TIM1_BKR_AOE|TIM1_BKR_OSSI|TIM1_BKR_BKE;
	TIM1->OISR = 0;
	TIM1->CR1 |= TIM1_CR1_CEN;
 	TIM1->BKR |= TIM1_LOCKLEVEL_3;
}

inline static void adc_cfg()
{
	ADC1->CR1 = ADC1_PRESSEL_FCPU_D18;
	ADC1->CR2 = ADC1_ALIGN_RIGHT;
  ADC1->CSR = 1; //channel sel

	ADC1->TDRL = 0x03; //smtt trg on adc pins is off
	ADC1->CR1 |= ADC1_CR1_ADON;
}
inline static void iwdg_cfg()
{
IWDG_WriteAccessCmd(IWDG_WriteAccess_Enable);
IWDG_SetPrescaler(IWDG_Prescaler_4);
IWDG_SetReload(0);
IWDG_Enable();
IWDG_ReloadCounter();
}

void convert_data (u16 d)
{
u8 k;
  for (k=0;k<4;k++)
  {
   display_buffer[k]=d%10;
   d/=10;
  }
}

void display_data ()
{
  static u8 pc;

  (pc)?(pc--):(pc = 3);
  N0=N1=N2=N3=0; DP=1;

  switch (display_buffer[pc])
 {
  case 0: A=0; B=0; C=0; D=0; E=0; F=0; G=1; break;
  case 1: A=1; B=0; C=0; D=1; E=1; F=1; G=1; break;
  case 2: A=0; B=0; C=1; D=0; E=0; F=1; G=0; break;
  case 3: A=0; B=0; C=0; D=0; E=1; F=1; G=0; break;
  case 4: A=1; B=0; C=0; D=1; E=1; F=0; G=0; break;
  case 5: A=0; B=1; C=0; D=0; E=1; F=0; G=0; break;
  case 6: A=0; B=1; C=0; D=0; E=0; F=0; G=0; break;
  case 7: A=0; B=0; C=0; D=1; E=1; F=1; G=1; break;
  case 8: A=0; B=0; C=0; D=0; E=0; F=0; G=0; break;
  case 9: A=0; B=0; C=0; D=0; E=1; F=0; G=0; break;
  case ' ': A=1; B=1; C=1; D=1; E=1; F=1; G=1; break;
  case 'u': A=1; B=1; C=0; D=0; E=0; F=1; G=1; break;
  case 'c': A=1; B=1; C=1; D=0; E=0; F=1; G=0; break;
  case 'b': A=1; B=1; C=0; D=0; E=0; F=0; G=0; break;
  case 't': A=1; B=1; C=1; D=0; E=0; F=0; G=0; break;
  case 'l': A=1; B=1; C=1; D=0; E=0; F=1; G=1; break;
  case 'o': A=1; B=1; C=0; D=0; E=0; F=1; G=0; break;
  case 'A': A=0; B=0; C=0; D=1; E=0; F=0; G=0; break;
  case 'H': A=1; B=0; C=0; D=1; E=0; F=0; G=0; break;
  case 'C': A=0; B=1; C=1; D=0; E=0; F=0; G=1; break;
  case 'P': A=0; B=0; C=1; D=1; E=0; F=0; G=0; break;
  case 'E': A=0; B=1; C=1; D=0; E=0; F=0; G=0; break;
  case 'U': A=1; B=0; C=0; D=0; E=0; F=0; G=1; break;
  case 'F': A=0; B=1; C=1; D=1; E=0; F=0; G=0; break;
  case 'L': A=1; B=1; C=1; D=0; E=0; F=0; G=1; break;
  case 'r': A=1; B=1; C=1; D=1; E=0; F=1; G=0; break;
  case '_': A=1; B=1; C=1; D=0; E=1; F=1; G=1; break;
  case 'd': A=1; B=0; C=0; D=0; E=0; F=1; G=0; break;
  case 'n': A=1; B=1; C=0; D=1; E=0; F=1; G=0; break;
  default:  A=1; B=1; C=1; D=1; E=1; F=1; G=0; break; //-
 }

   switch (pc)
 {
  case 0: N0=1; if(dppos == 0) {DP=0;} break;
  case 1: N1=1; if(dppos == 1) {DP=0;} break;
  case 2: N2=1; if(dppos == 2) {DP=0;} break;
  case 3: if(display_buffer[3]){N3=1;} if(dppos == 3) {DP=0;} break;
  default: DP=1; break;
 }
 }

void delay(volatile uint16_t nCount)
{
    while (nCount)
    {
        nCount--;
        IWDG_ReloadCounter();
    }
}

void scan_keys ()
{
#define lp  150
#define lh  150

static u8 count[2];

if (!key) 	{count[0]++; }
else 			  {count[0]=count[1]=0; btn=rls;}

if ((count[0]>lp)&&(key!=hold))	{btn=prs;count[0]=0;count[1]++;}
if (count[1]>lh) 						    {btn=hold;count[1] = 0;}
}

void dev_mgmt(); u8 check_bat();
void pwm_null(bool);

INTERRUPT_HANDLER(TIM1_X3_IRQ, 11)
{
const u8 rcnt0def=0xff;
const u8 rcnt1def=0x0f;
static u8 rcnt0,rcnt1;
static u16 tmp1=0;
u8 adch,adcl;

  if(!rcnt0)
  { //Vbat
    ADC1->CSR=0;
    rcnt0=rcnt0def;

    if(rcnt1)
    {
      rcnt1--;
      adcl = ADC1->DRL;
      adch = ADC1->DRH;
      tmp1+=adcl|adch<<8;
    }
    else {rcnt1=rcnt1def; adc1val=tmp1/(rcnt1def); tmp1=0;}
  }
  else
  { //Vout
    static u8 tmp=0;

    rcnt0--;if(!rcnt0){ADC1->CSR = 1;}
      adcl = ADC1->DRL;
      adch = ADC1->DRH;
    adc0val=adcl|adch<<8;

	 if(pge)
	 	{
    	if(adc0val<apv){if(tmp<arrv/2){tmp++;}else{tmp=arrv/2;if(ovl<0xff)ovl++;}}
    	else{if(tmp){tmp--;} ovl=0;}
    	TIM1->CCR1L=TIM1->CCR2L=tmp;
    	TIM1->CCR3L=TIM1->CCR4L=arrv-tmp;
  		}
	 else
	 {
		TIM1->CCR1L=TIM1->CCR2L=0;
    TIM1->CCR3L=TIM1->CCR4L=arrv;
	 }
   }

    if(tim0)tim0--;
    if(tim1)tim1--;
    if(tim2)tim2--;
    if(tim3)tim3--;

	IWDG_ReloadCounter();
  ADC1->CR1 |= ADC1_CR1_ADON;
	TIM1->SR1 = 0;
}

void main()
{
gpio_cfg();
clk_cfg();
adc_cfg();
t1_cfg();
iwdg_cfg();
pwm_off; relay=0;

BEEP_Init(BEEP_FREQUENCY_2KHZ);
BEEP_Cmd(DISABLE);

ADC1->CR1 |= ADC1_CR1_ADON;

asm("rim");

  while(1)
  {
		dev_mgmt();
		scan_keys ();
    display_data();
  }
}

void assert_failed(uint8_t* file, uint32_t line)
{

}

void dev_mgmt()
{
 static u8 bt;
 if(btn==rls){bt=0;}

 switch (dstate)
 {

  case normal:
  {
    if ((btn==hold)&&(!bt)) {dstate=sdn; bt=1;}
    if(!tim0)
    {
      tim0=0xffff/2;
      convert_data((u16)((u32)adc1val*u_max_adc/(0x3ff*10)));
      dppos=1;
    }

    if(!check_bat())	{dstate=lobat;}
    if(ovl==0xff)		{dstate=undervoltage;}
  }
  break;

  case started:
  {
    if((btn==hold)&&(!bt)){dstate=sdn;bt=1;}

	  if(!tim0)
      {
        tim0=0xffff/4;
        convert_data((u16)((u32)adc1val*u_max_adc/(0x3ff*10)));
        display_buffer[3]='o';
        dppos=1;   
       
        if(check_bat())
        {
          BEEP_Cmd(ENABLE);
          delay(65000);
          BEEP_Cmd(DISABLE);
          pwm_on; relay=1; dppos=1;
          dstate=normal;
        }
      }
   }
  break;

  case undervoltage:
  {
  static u8 t=0;
  if(!ovl) {dstate=normal;t=0;tim2=0;}
  else if((!tim2)&&(!t)){tim2=0xffff;t=1;}
  else if((!tim2)&&(t)) {t=0; dstate=sdn;}
  else
  {
   display_buffer[3]='L';
   display_buffer[2]='o';
   display_buffer[1]='o';
   display_buffer[0]='U';
   dppos=1;
  }

  }
  break;

  case overvoltage:
  {
  static u8 n;
  if((!tim2)&&(!n)){tim2=0xffff;n=1;}
  else if((!tim2)&&(n)) {n=0; dstate=sdn;}
  else
  {
   display_buffer[3]='E';
   display_buffer[2]='c';
   display_buffer[1]='c';
   display_buffer[0]='U';
   dppos=3;
  }
  }
  break;

  case lobat:
  {
  static u8 lbtr=0, lbcnt, bb=1;

  if(!lbtr) 
  {
   tim1=0xffff;lbtr=1;lbcnt=0xff;
   display_buffer[3]='L';
   display_buffer[2]='o';
   display_buffer[1]='b';
   display_buffer[0]='t';
   dppos=5;  
  }
  if(check_bat()){tim1=lbtr=lbcnt=0; dstate=normal;break;}
  
  if((!tim1)&&((lbcnt)))
  {
    tim1=0xffff;lbcnt--;
    bb^=1;
    if(bb)BEEP_Cmd(ENABLE);else BEEP_Cmd(DISABLE);
    if(90>((u16)((u32)adc1val*u_max_adc/(0x3ff*10))))
    {lbtr=tim1=lbcnt=0; dstate=sleep; BEEP_Cmd(DISABLE); break;}
  }
  else if((!tim1)&&(!lbcnt)) {lbtr=0; dstate=sleep; BEEP_Cmd(DISABLE); break;}
  }
  break;

  case sdn:
  {
  pwm_off; relay=0;
  {
   display_buffer[3]='_';
   display_buffer[2]='_';
   display_buffer[1]='_';
   display_buffer[0]='_';
   dppos=5;
  }
  if ((btn==hold)&&(!bt)){dstate=started;bt=1;}
  }
  break;

  case sleep:
  {

  if(check_bat()){if(((btn==prs)||(btn==hold))&&(!bt))dstate=started;bt=1;}
  else if ((btn==prs)||(btn==hold)){tim2=0xffff;}
  else if (tim2)
  {
   display_buffer[3]='A';
   display_buffer[2]='d';
   display_buffer[1]= 1;
   display_buffer[0]='u';
   dppos=5;
  }
  else {dstate=sdn;}
  }
  break;

  default: break;
 }
}
u8 check_bat()
{
 static u16 tmp0,tmp1;
 static u8 btrg;
 if(tmp0!=adc1val)
 {
 tmp0=adc1val;
 tmp1=(u16)((u32)tmp0*u_max_adc/(0x3ff*10));
 }
 if(tmp1>=120){btrg=1; return 1;}

 if((tmp1>108)&&(btrg))return 1;
 else if(tmp1<=108){btrg=0;return 0;}
 else return 0;

}
void pwm_null(bool t)
{
	pge=0;

}